r"""
# -----------------------------------------------------------------------------
# This WebServer aims to provide a good starting point when creating
# wslink based service.
#
# $ python ./server/server.py --content ./www
# -----------------------------------------------------------------------------
"""

import argparse

from api import PubSubAPI

from wslink import server
from wslink.websocket import ServerProtocol

# -----------------------------------------------------------------------------


class WebServer(ServerProtocol):
    authKey = "wslink-secret"

    @staticmethod
    def add_arguments(parser):
        parser.add_argument(
            "--virtual-env", default=None, help="Path to virtual environment to use"
        )

    @staticmethod
    def configure(options):
        WebServer.authKey = options.authKey

    def initialize(self):
        self.registerLinkProtocol(PubSubAPI())

        # Update authentication key to use
        self.updateSecret(WebServer.authKey)


# -----------------------------------------------------------------------------
# Main
# -----------------------------------------------------------------------------

if __name__ == "__main__":
    # Create argument parser
    parser = argparse.ArgumentParser(description="WSLink sample application")

    # Add arguments
    server.add_arguments(parser)
    WebServer.add_arguments(parser)
    args = parser.parse_args()
    WebServer.configure(args)

    # Start web server
    server.start(None, WebServer)



from myProtocol import MyProtocol

from wslink import server
from wslink.websocket import ServerProtocol


class ExampleServer(ServerProtocol):
    authKey = "wslink-secret"

    @staticmethod
    def configure(options):
        ExampleServer.authKey = options.authKey

    def initialize(self):
        self.registerLinkProtocol(MyProtocol())

        # Update authentication key to use
        self.updateSecret(ExampleServer.authKey)


def simple_start(argv=None):
    server.start(argv, ExampleServer)


if __name__ == "__main__":
    simple_start()



import asyncio

import pytest

from wslink.emitter import EventEmitter


def _create_listeners(emitter):
    payloads = {}

    def on_foo(val):
        payloads["foo"] = val

    async def on_bar(val):
        await asyncio.sleep(0.05)
        payloads["bar"] = val

    emitter.add_event_listener("foo", on_foo)
    emitter.add_event_listener("bar", on_bar)

    return payloads, on_foo, on_bar


@pytest.mark.asyncio
async def test_add_listener():
    emitter = EventEmitter()
    payloads, on_foo, on_bar = _create_listeners(emitter)

    emitter.emit("foo", 0)
    emitter.emit("bar", 1)

    await asyncio.sleep(0.1)

    assert payloads.get("foo") == 0
    assert payloads.get("bar") == 1


@pytest.mark.asyncio
async def test_remove_listeners():
    emitter = EventEmitter()
    payloads, on_foo, on_bar = _create_listeners(emitter)

    emitter.remove_event_listener("bar", on_bar)

    emitter.emit("foo", 0)
    emitter.emit("bar", 1)

    await asyncio.sleep(0.1)

    assert payloads.get("foo") == 0
    assert payloads.get("bar") is None


@pytest.mark.asyncio
async def test_clear_listeners():
    emitter = EventEmitter()
    payloads, on_foo, on_bar = _create_listeners(emitter)

    emitter.clear()

    emitter.emit("foo", 0)
    emitter.emit("bar", 1)

    await asyncio.sleep(0.1)

    assert payloads.get("foo") is None
    assert payloads.get("bar") is None


def test_event_type_runtime():
    emitter = EventEmitter(allowed_events=("foo", "bar"))

    emitter.emit("foo")
    emitter.emit("bar")

    with pytest.raises(ValueError):
        emitter.emit("baz")

    assert emitter.allowed_events == {"foo", "bar"}


@pytest.mark.asyncio
async def test_event_emit_call():
    emitter = EventEmitter(allowed_events=("foo", "bar"))
    payloads, on_foo, on_bar = _create_listeners(emitter)

    assert emitter.allowed_events == {"foo", "bar"}

    emitter("foo", 0)
    emitter("bar", 1)

    with pytest.raises(ValueError):
        emitter("baz", 2)

    await asyncio.sleep(0.1)

    assert payloads.get("foo") == 0
    assert payloads.get("bar") == 1


@pytest.mark.asyncio
async def test_event_emit_attribute():
    emitter = EventEmitter(allowed_events=("foo", "bar"))
    payloads, on_foo, on_bar = _create_listeners(emitter)

    assert emitter.allowed_events == {"foo", "bar"}

    emitter.foo(0)
    emitter.bar(1)

    with pytest.raises(AttributeError):
        emitter.baz()

    await asyncio.sleep(0.1)

    assert payloads.get("foo") == 0
    assert payloads.get("bar") == 1



from pathlib import Path

from wslink import register as exportRPC
from wslink import schedule_callback
from wslink.websocket import LinkProtocol

# -----------------------------------------------------------------------------
# WS protocol definition
# -----------------------------------------------------------------------------


class MyProtocol(LinkProtocol):
    def __init__(self):
        super().__init__()
        self.loopTask = None
        self.topic = "image"
        self.subscribers = 0
        self.subMsgCount = 0

    @exportRPC("myprotocol.add")
    def add(self, listOfNumbers):
        if type(listOfNumbers) is list:
            result = 0
            for value in listOfNumbers:
                result += value
            return result

        # How should a client return an error? Probably throw()
        print("Unexpected arg", listOfNumbers)
        return 0

    @exportRPC("myprotocol.mult")
    def mult(self, listOfNumbers):
        if type(listOfNumbers) is list:
            result = 1
            for value in listOfNumbers:
                result *= value
            return result

        print("Unexpected arg", listOfNumbers)
        return 0

    @exportRPC("myprotocol.image")
    def image(self, alt=False):
        filename = "kitware.png" if not alt else "kitware2.png"
        filename = Path(__file__).with_name(filename)
        return {"blob": self.addAttachment(filename.read_bytes())}

    @exportRPC("myprotocol.unwrapped.image")
    def bareImage(self, alt=False):
        filename = "kitware.png" if not alt else "kitware2.png"
        filename = Path(__file__).with_name(filename)
        return self.addAttachment(filename.read_bytes())

    def pushImage(self):
        print("push image", self.subMsgCount)
        msg = self.image(self.subMsgCount % 2 == 0)
        # publish binary message.
        self.publish(self.topic, msg)
        self.subMsgCount += 1

        self.loopTask = schedule_callback(2, self.pushImage)

    @exportRPC("myprotocol.postbinary")
    def postBinary(self, data):
        return f"received binary data of length {len(data)}"

    @exportRPC("myprotocol.stream")
    def startStream(self, topic):
        print("start", topic)
        # set up repeated send of images until unsubscribed.
        if not self.loopTask:
            self.loopTask = schedule_callback(0, self.pushImage)

        self.topic = topic
        self.subscribers += 1
        return {"subscribed": topic}

    @exportRPC("myprotocol.stop")
    def stopStream(self, topic):
        print("stop", topic)
        if self.topic == topic and self.subscribers > 0:
            self.subscribers -= 1
            if self.subscribers == 0 and self.loopTask:
                print("canceling loop task")
                self.loopTask.cancel()
                self.loopTask = None
            return {"unsubscribed": topic}
        return 0

    # test nesting attachments
    @exportRPC("myprotocol.nested.image")
    def testNesting(self):
        img = self.image()
        # just using 'bytes' only works in Py3
        bytes_list1 = bytes(bytearray([1, 2, 3, 4]))
        bytes_list2 = bytes(bytearray([5, 6, 7, 8, 9, 10]))
        return {
            "image": img,
            "bytesList": [
                self.addAttachment(bytes_list1),
                self.addAttachment(bytes_list2),
            ],
        }

    # test NaN and infinity
    @exportRPC("myprotocol.special")
    def testSpecials(self, *_):
        return [float("inf"), float("nan"), float("-inf")]



from wslink import register as exportRpc
from wslink import schedule_callback
from wslink.websocket import LinkProtocol

MESSAGE_LIST = [
    "Nice to meet you",
    "What's your name?",
    "Mine is wslink.py",
]


class PubSubAPI(LinkProtocol):
    def __init__(self, **_):
        super().__init__()
        self.topic = "wslink.communication.channel"
        self.msgIdx = 0
        self.keepTalking = True
        self.frequency = 5  # 5 seconds
        self.startTalking()

    def saySomething(self):
        if not self.keepTalking:
            return

        if self.msgIdx + 1 < len(MESSAGE_LIST):
            self.msgIdx += 1
        else:
            self.msgIdx = 0

        self.publish(self.topic, MESSAGE_LIST[self.msgIdx])

        if self.keepTalking:
            schedule_callback(self.frequency, self.saySomething)

    @exportRpc("wslink.say.hello")
    def sayHello(self, message):
        print("sayHello", message)
        msgToPost = f"py server: {message}"
        self.publish(self.topic, msgToPost)
        return msgToPost

    @exportRpc("wslink.start.talking")
    def startTalking(self):
        self.keepTalking = True
        self.saySomething()

    @exportRpc("wslink.stop.talking")
    def stopTalking(self):
        self.keepTalking = False
        self.msgIdx = 0



def test_import_root():
    import wslink  # noqa: F401


def test_import_chunking():
    import wslink.chunking  # noqa: F401


def test_import_emitter():
    import wslink.emitter  # noqa: F401


def test_import_launcher():
    import wslink.launcher  # noqa: F401


def test_import_protocol():
    import wslink.protocol  # noqa: F401


def test_import_publish():
    import wslink.publish  # noqa: F401


def test_import_relay():
    import wslink.relay  # noqa: F401


def test_import_server():
    import wslink.server  # noqa: F401


def test_import_ssl():
    import wslink.ssl_context  # noqa: F401


def test_import_uri():
    import wslink.uri  # noqa: F401


def test_import_websocket():
    import wslink.websocket  # noqa: F401


def test_import_backends():
    import wslink.backends  # noqa: F401


def test_import_backends_aiohttp():
    import wslink.backends.aiohttp  # noqa: F401


def test_import_backends_generic():
    import wslink.backends.generic  # noqa: F401
